# ============================================================================
# import
# ============================================================================

from ctypes import c_uint16, c_uint32, c_uint64, c_uint8
import sys
import time
import inspect
import usb.util

#import os
#os.environ['PYUSB_DEBUG'] = 'debug'

import SUSlv

# ============================================================================
# 本体
#
# main() 内の下記いずれかを選んでターゲットボードを指定してください
#
# dev = SUSlv.UsbOpen(SUSlv.const.PID_KX_Card7)      # KX-Card7 
# dev = SUSlv.UsbOpen(SUSlv.const.PID_CX_Card5)      # CX-Card5
# dev = SUSlv.UsbOpen(SUSlv.const.PID_AX_Card7)      # AX-Card7
# dev = SUSlv.UsbOpen(SUSlv.const.PID_AX_Card7M)     # AX-Card7M
# dev = SUSlv.UsbOpen(SUSlv.const.PID_SX_Card7)      # SX-Card7
# dev = SUSlv.UsbOpen(SUSlv.const.PID_ZQ_Card)       # ZQ-Card
# ============================================================================


def main():
    # 使用するボードを指定してください。
    # USBコンフィグをする場合、USBコンフィグテスト部のファイル名とFPGA種別を指定してください。
    dev = SUSlv.UsbOpen(SUSlv.const.PID_AX_Card7)    # AX-Card7 
    #dev = SUSlv.UsbOpen(SUSlv.const.PID_CX_Card5)    # CX-Card5 

    # デバイスオープンできたか確認
    if dev is None:
        raise ValueError('Device not found')
        exit(-1)


    # ==========================
    # テスト実施
    # ==========================

    # FPGA コンフィグ
    test_USB_config(dev)

    # ボードの情報表示
    show_boardInfo(dev)

    # レジスタアクセス(A0コマンド)
    #test_Register_access_A0(dev)

    # レジスタアクセス(B0コマンド)
    test_Register_access_B0(dev)

    # データ転送テスト
    test_Data_Transfer(dev)

    # FPGA デコンフィグ
    # test_USB_deconfig(dev)

    # デバイスをクローズする
    ret = SUSlv.UsbClose(dev)
    return


# ============================================================================
# プリント関連小物
# ============================================================================
def dumpByteArray(ary):
    for i in ary:
        print("0x%02X " % (i), end='')


# ============================================================================
# ボード情報を表示する
# ============================================================================
def show_boardInfo(dev: usb.core.Device):
    print("")
    print("[", sys._getframe().f_code.co_name, "]")

    ret = SUSlv.get_venderRequest_boardInfo(dev, 0x0040)
    productID = ret[1] << 8 | ret[0]
    print('productID = 0x{:04X}'.format(productID))

    ret = SUSlv.get_venderRequest_boardInfo(dev, 0x0080)
    chipRevision = ret[1] << 8 | ret[0]
    print('chipRevision = 0x{:04X}'.format(chipRevision))

    ret = SUSlv.get_venderRequest_boardInfo(dev, 0x00C0)
    firmwareVersion = ret[1] << 8 | ret[0]
    print('firmwareVersion = 0x{:04X}'.format(firmwareVersion))

    ret = SUSlv.get_venderRequest_boardInfo(dev, 0x0000)
    connectionSpeed = ret[1] << 8 | ret[0]
    print('connectionSpeed = 0x{:04X}'.format(connectionSpeed))

    return


# ============================================================================
# B0コマンドによる レジスタライトとリード
#
# # レジスタ構成（2台とも共通）
# No.0（64bit R/W)　値の操作（メモリ操作時は"0"設定（初期値）のこと）
# No.2（8bit R/W)　1でLED点灯します。
# No.3（8bit R)　ディップSWとタクトスイッチ操作
# No.4（16bit R/W)　CX-Card5では7セグ表示、AX-Card7Mでは値の操作
# No.5（32bit R/W)　値の操作
# ============================================================================


def test_Register_access_B0(dev: usb.core.Device):
    print("")
    print("[", sys._getframe().f_code.co_name, "]")

    reg_addr: c_uint32

    # registor0 64bit 読み書き
    print("")
    print("registor 0 64bit write/read(verify)")
    reg_addr = 0
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_64BIT, 0x0000000000000000)
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_64BIT, 0xffffffffffffffff)
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_64BIT, 0x5a5a5a5a5a5a5a5a)
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_64BIT, 0xa5a5a5a5a5a5a5a5)
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_64BIT, 0x0000000000000000)

    # registor2 LED点滅
    print("")
    print("registor 2 LED blink")
    reg_addr = 2
    data: c_uint8 = 0
    for i in range(3):
        data = 0x55
        wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_8BIT,data)
        time.sleep(0.5)
        data = 0xAA
        wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_8BIT,data)
        time.sleep(0.5)

    # register3 SW入力
    print("")
    print("registor 3 switch-input")
    reg_addr = 3
    for i in range(10):
        ret = wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_8BIT, None)
        time.sleep(0.25)

    # registor4 16bit 読み書き
    print("")
    print("registor 4 16bit write/read(verify)")
    reg_addr = 4
    _7segData: c_uint16 = 1
    for i in range(16):
        wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_16BIT, _7segData)
        time.sleep(0.5)
        _7segData <<= 1
    _7segData = 0xffff
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_16BIT, _7segData)

    # registor6 32bit 読み書き
    print("")
    print("registor 6 32bit write/read(verify)")
    reg_addr = 6
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_32BIT, 0xccbbaa99)
    wr_regB0(dev, reg_addr, SUSlv.const.SUSlv_RANGE_32BIT, 0x00ffeedd)

    return


def dumpHex_nBIT(val, accessType):
    if(accessType == SUSlv.const.SUSlv_RANGE_8BIT):
        print("0x%02X " % (val), end='')
    elif(accessType == SUSlv.const.SUSlv_RANGE_16BIT):
        print("0x%04X " % (val), end='')
    elif(accessType == SUSlv.const.SUSlv_RANGE_32BIT):
        print("0x%08X " % (val), end='')
    elif(accessType == SUSlv.const.SUSlv_RANGE_64BIT):
        print("0x%016X " % (val), end='')
    else:
        print(val)


def wr_regB0(dev: usb.core.Device, reg_addr, accessType, data):
    if(data != None):
        print("Reg_Write_B0(%d) = " % (reg_addr), end='')
        dumpHex_nBIT(data, accessType)
        print("")
        ret = SUSlv.Reg_Write_B0(dev, reg_addr, accessType, data)

    ret = SUSlv.Reg_Read_B0(dev, reg_addr, accessType)
    print("Reg_Read_B0 (%d) = " % (reg_addr), end='')
    dumpHex_nBIT(ret, accessType)

    if(data != None):
        if(data != ret):
            print(" ... unmatched",end='' )
            return
        print(" ... verify ok",end='' )
    print("")


# ============================================================================
# A0コマンドによる レジスタライトとリード
# ============================================================================


def test_Register_access_A0(dev: usb.core.Device):
    print("")
    print("[", sys._getframe().f_code.co_name, "]")
    reg_addr = 0


    # USB productID を取得
    ret = SUSlv.get_venderRequest_boardInfo(dev, 0x0040)
    productID = ret[1] << 8 | ret[0]


    if(productID == SUSlv.const.PID_CX_Card5):
        # registor0 32bit 読み書き 64bitアクセスだとコンペア不一致になる
        print("")
        print("registor 0 32bit write/read(verify)")
        reg_addr = 0
        wr_regA0(dev, reg_addr, [0x01, 0x23, 0x45, 0x67])
        wr_regA0(dev, reg_addr, [0x11, 0x22, 0x33, 0x44])
    elif(productID == SUSlv.const.PID_AX_Card7M):
        # registor0 64bit 読み書き
        print("")
        print("registor 0 64bit write/read(verify)")
        reg_addr = 0
        wr_regA0(dev, reg_addr, [0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef])
        wr_regA0(dev, reg_addr, [0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88])



    # registor2 LED点滅
    print("")
    print("registor 2 LED blink")
    reg_addr = 2
    for i in range(3):
        wr_regA0(dev, reg_addr, [0x55])
        time.sleep(0.5)
        wr_regA0(dev, reg_addr, [0xAA])
        time.sleep(0.5)

    # register3 SW入力
    print("")
    print("registor 3 switch-input")
    reg_addr = 3
    for i in range(10):
        ret = SUSlv.Reg_Read(dev, reg_addr, 1)
        print("Reg_Read (%d) = " % (reg_addr), end='')
        dumpByteArray(ret)
        time.sleep(0.25)
        print("")

    # registor4 16bit 読み書き
    print("")
    print("registor 4 16bit write/read(verify)")
    reg_addr = 4
    _7segData: c_uint16 = 1
    for i in range(16):
        d = SUSlv.uint16_to_uint8(_7segData)
        wr_regA0(dev, reg_addr, d)
        time.sleep(0.5)
        _7segData <<= 1
    _7segData = 0xffff
    d = SUSlv.uint16_to_uint8(_7segData)
    wr_regA0(dev, reg_addr, d)

    # registor6 32bit 読み書き
    print("")
    print("registor 6 32bit write/read(verify)")
    reg_addr = 6
    wr_regA0(dev, reg_addr, [0x99, 0xaa, 0xbb, 0xcc])
    wr_regA0(dev, reg_addr, [0xdd, 0xee, 0xff, 0x00])

    return


def wr_regA0(dev: usb.core.Device, reg_addr, data):
    print("Reg_Write(%d) = " % (reg_addr), end='')
    dumpByteArray(data)
    print("")
    ret = SUSlv.Reg_Write(dev, reg_addr, data)

    ret = SUSlv.Reg_Read(dev, reg_addr, len(data))
    print("Reg_Read (%d) = " % (reg_addr), end='')
    dumpByteArray(ret)

    for i in range(len(data)):
        if(data[i] != ret[i]):
            print(" ... unmatched")
            return
    print(" ... verify ok")


# ============================================================================
# メモリ転送テスト
# ============================================================================
#
# 【設定項目】
#
# 転送先 RAMの指定
#   ram_select
#
# テスト入力ファイルを指定　下記のどちらか片方を 1、 他を0 とすること
#   use_input_fromFile = 0          # ファイルからテストデータを読み出す
#   use_input_serialValue = 1       # テストデータを生成する
#
# テストデータファイル名
#   testDataFilename
#
# テストデータ(順番になっている uint32値)を生成し使う場合。
#   arraySz = 1024*1024*4           # 4MB のテストデータを作ってWRITE/READする
#
# 4byte 単位の転送以外を試す場合は データを1〜3バイト削ってみる
#   hanpa_byte_test = 3             # 3byte削る場合
# ============================================================================
def test_Data_Transfer(dev: usb.core.Device):
    print("")
    print("[", sys._getframe().f_code.co_name, "]")

    # ターゲットによる転送先・転送サイズの調整
    ram_select = 0
    ramsizeMax = 0

    # ===================================================================
    # テスト入力ファイルを指定　下記のどちらか片方を 1、 他を0 とすること
    # ===================================================================
    testDataFilename = "Random1_512MB.bin"  # テストデータファイル名 
    use_input_fromFile = 1          # ファイルからテストデータを読み出す
    use_input_serialValue = 0       # テストデータを生成する

    # reg0 = 0 先頭アドレス設定
    reg_addr = 0
    inputData = 0x0
    ret = SUSlv.Reg_Write_B0(
        dev, reg_addr, SUSlv.const.SUSlv_RANGE_64BIT, inputData)

    # テスト入力を バイナリファイルから読む場合
    # ファイルサイズは4の倍数であること
    if(use_input_fromFile):
        print("loading ", testDataFilename)
        time_start = time.time()
        with open(testDataFilename, "rb") as f:
            dataF = f.read()
        inputData = bytearray(dataF)
        length_of_uint8 = len(inputData)
        f.close()
        time_diff = time.time() - time_start
        print("time = %.1f[sec] speed = %.1f[MB/sec]" %
              (time_diff, length_of_uint8 / time_diff / (1024*1024)))

    # テストデータ(順番になっている uint32値)を生成し使う場合。
    # データサイズは arraySz に入れる
    if(use_input_serialValue):
        arraySz = 4*1024*1024   # arraySz は 4の倍数であること
        inputData: c_uint8 = [0] * (arraySz)
        for i in range(0, arraySz, 4):
            d = SUSlv.uint32_to_uint8(i)
            inputData[i+0] = d[0]
            inputData[i+1] = d[1]
            inputData[i+2] = d[2]
            inputData[i+3] = d[3]

    # USB productID を取得
    ret = SUSlv.get_venderRequest_boardInfo(dev, 0x0040)
    productID = ret[1] << 8 | ret[0]

    if(productID == SUSlv.const.PID_CX_Card5):
        # =====================================================================
        # SDRAMを選択の 1つめを選択
        # ram_select
        #   0   FPGA内部 8kB SRAM
        #   1   DDR3-SDRAM(512MB)を選択
        # SDRAMを選択
        # =====================================================================
        ram_select = 1
        ret = SUSlv.Reg_Write_B0(
            dev, 1, SUSlv.const.SUSlv_RANGE_8BIT, ram_select)
        ramsizeMax = 512*1024*1024
        if(ram_select == 0):
            ramsizeMax = 8*1024
        del inputData[ramsizeMax:]

    elif(productID == SUSlv.const.PID_AX_Card7):
        # =====================================================================
        # テスト対象 RAMを選択
        # ram_select
        #   0    FPGA内部128kB SRAM
        #   1    DDR3-SDRAMを選択
        # =====================================================================
        ram_select = 1
        ret = SUSlv.Reg_Write_B0(
            dev, 1, SUSlv.const.SUSlv_RANGE_8BIT, ram_select)
        ramsizeMax = 512*1024*1024
        if(ram_select == 0):
            ramsizeMax = 128*1024
        del inputData[ramsizeMax:]
    else:
        return False

    # =========================================================================
    # 4byte 単位の転送以外を試す場合は データを1〜3バイト削ってみる
    # =========================================================================
    hanpa_byte_test = 3    # 3byte 削ってみた
    #del inputData[hanpa_byte_test]

    # 入力データサイズ確定
    length_of_uint8 = len(inputData)

    # データ書き込み
    print("SUSlv.DataWrite(0x%08x)..." % (length_of_uint8), end='')
    time_start = time.time()
    ret = SUSlv.Data_Write(dev, inputData)
    time_diff = time.time() - time_start
    print("ret  = %d(0x%X)" % (ret, ret), end='')
    print("\ttime = %.1f[sec] speed = %.1f[MB/sec]" %
          (time_diff, length_of_uint8 / time_diff / (1024*1024)))

    # データ読み込み
    print("SUSlv.DataRead (0x%08X)..." % (length_of_uint8), end='')
    time_start = time.time()
    result, transferd_bytes = SUSlv.Data_Read(dev, length_of_uint8)
    time_diff = time.time() - time_start
    print("ret  = %d(0x%X)" % (transferd_bytes, transferd_bytes), end='')
    print("\ttime = %.1f[sec] speed = %.1f[MB/sec]" %
          (time_diff, length_of_uint8 / time_diff / (1024*1024)))

    # コンペア
    print("Compare wrote-read bytes...")
    print("length of write = ", len(inputData))
    print("length of read  = ", len(result))

    if(len(inputData) != len(result)):
        print("Data length mismatch.")
        return False

    for i in range(length_of_uint8):
        # if(0 == (i % 0x100000)):
        # print("Comparing 0x%08x 0x%08x bytes left." %
        #       (i, length_of_uint8 - i))
        if(inputData[i] != result[i]):
            print("Mismatched value. data[%d]=%d , result[%d]=%d"
                  % (i, inputData[i], i, result[i]))
            return False

    print("All %d bytes have been matched." % (length_of_uint8))
    return True


# ============================================================================
# USBコンフィグ テスト
# ============================================================================
def test_USB_config(dev: usb.core.Device):
    print("")
    print("[", sys._getframe().f_code.co_name, "]")

    # USB productID を取得
    ret = SUSlv.get_venderRequest_boardInfo(dev, 0x0040)
    productID = ret[1] << 8 | ret[0]

    fn = "AX-Card7_GPIF2_Verilog.bin"
    #fn = "CX-Card5_GPIF2_Verilog.rbf"

    print("FPGA configuration start with '%s'" % (fn))

    # FPGA: Altera=1, Xilinx=0
    ret = SUSlv.Usb_Confing(dev, fn, 0)
    if(ret == False):
        print("Usb_Confing(dev,%s) ret = %d" % (fn, ret))
        return False

    print("FPGA configuration completed successfully.")


# ============================================================================
# USBデコンフィグ テスト
# ============================================================================
def test_USB_deconfig(dev: usb.core.Device):
    print("FPGA deconfiguration start")
    ret = SUSlv.Usb_Deconfing(dev)
    if(ret == False):
        print("Usb_Deconfing(dev) ret = %d" % (ret))
        return False
    print("FPGA deconfiguration completed successfully.")


# ============================================================================
# main() 呼び出し
# ============================================================================
if __name__ == "__main__":
    main()
